/* The Computer Language Benchmarks Game
   http://shootout.alioth.debian.org/

   contributed by Greg Buchholz (original iterative version)
   modified by Yannick Gingras (made tail recursive)
*/

#include <stdio.h>
#include <stdlib.h>

#include "bmsg.h"

#include "ptime.h"

int iterate(double Zr, double Zi,
	    double Tr, double Ti,
	    double Cr, double Ci,
	    int iter)
{

  if (iter == 0)
    return 1;

  Zi = 2.0 * Zr * Zi + Ci;
  Zr = Tr - Ti + Cr;
  Tr = Zr * Zr;
  Ti = Zi * Zi;

  if (Tr + Ti > 4.0)
    return 0;

  return iterate (Zr, Zi, Tr, Ti, Cr, Ci, iter - 1);
}

int main (int argc, char **argv)
{
  int w, h, buflen, rowlen, bit_num = 0, start, end;
  char byte_acc = 0;
  char *buf, *pbuf;
  double x, y, wrat, hrat;
  double Cr, Ci;
  struct connection_t *c;

  c = wait_for_connection(argv[1]);

  recv_byte_message(c, sizeof(w), &w);
  recv_byte_message(c, sizeof(start), &start);
  recv_byte_message(c, sizeof(end), &end);

  h = w;
  wrat = 2.0 / w;
  hrat = 2.0 / h;
  rowlen = (w / 8 + (w % 8 ? 1 : 0));
  buflen = rowlen * (end - start);
  buf = (char *) malloc (buflen);
  pbuf = buf;

  for (y = start; y < end; ++y)
    {
      Ci = y * hrat - 1.0;

      for (x = 0; x < w; ++x)
	{
	  Cr = x * wrat - 1.5;

	  byte_acc <<= 1;
	  if (iterate (0.0, 0.0, 0.0, 0.0, Cr, Ci, 50))
	    byte_acc |= 0x01;

	  if (++bit_num == 8)
	    {
	      *(pbuf++) = byte_acc;
	      bit_num = byte_acc = 0;
	    }
	}

      if (bit_num)
	{
	  byte_acc <<= (8 - w % 8);

	  *(pbuf++) = byte_acc;
	  bit_num = byte_acc = 0;
	}
    }

  send_byte_message(c, buflen, buf);

  printf("done %s %ld\n", argv[1], get_process_time());

  return 0;
}
